{ Demo program for ChroMathsome - Linear Algebra Pro.
  Below, you find example implementations of using new (and older) functions available in
  ChroMathsome - Linear Algebra and Linear Algebra Pro.

  This code was written during the development stages of ChroMathsome - Linear Algebra.
  There's no pre-written script followed so you may find (some of) this code
  (the order of implemented functions) and some of the variables a little strange
  (this is also the reason why this demo program is so LARGE ;-)).

  Note that, since ChroMathsome has its own pre-defined exceptions, you do not need to
  write supersofisticated exception handling routines.
  The only exceptions written here are to make sure the pointers get freed, even
  when an exception has occured (a try ... finally block).

  Since ChroMathsome - Linear Algebra automatically updates the dimensions of
  matrices and vectors (if needed), we gave most of the matrices/vectors in the
  initialization part (InitRealMatrix, ...) a dummy value.
  You are, however, encouraged to enter the correct values (if you know them at forehand)
  since this will make your code much more readable!
  For example:
  In the decomposition section, we wrote
      InitRealMatrix(M, 50, 50);
      InitRealMatrix(M1, 15, 15);
      InitRealMatrix(M2, 5, 5);
      ...
  It's obvious that both matrices MUST have other dimensions (which? ...).
  In this demo, we used our privilege to ignore this :-))

  When studying the code below, notice how nicely ChroMathsome - Linear Algebra
  keeps the user away from the implementation of matrices/vectors (information hiding principle).
  Matrices/vectors can be used as single datatypes.
  Example:

      (* Copy matrix M1 (including its dimensions) to matrix M -> M1 = M. *)
      Copy_RealMatrix(M, M1);
      RealMatrix_Inverse(M);  (* Invert matrix M *)
      Write_RealMatrix(M);
      Write_Message('Done. Testing result ... ');
      RealMatrix_Multiplication(M2, M1, M);  (* M2 = Original (M1) * inverse matrix (M) *)
      Write_Message('Done. Test result: ');
      Write_RealMatrix(M2);

  Imagine how much code it takes to write this from scratch ...
  Also, notice that the code can (almost) replace the pseudo-code that describes what
  you want to do!

  For this demo we used the RichEdit control as this (seems) to have no limit as for the
  amount of lines (and their length) it can handle
  (Delphi 1 version -> TMemo component: limit 32K and length 256 characters).

  In the code below, several lines are placed between comment delimiters.
  You may uncomment these lines but since they do not show a (for this demo) useful
  result, we have written them as comments.
  This will prevent the memo from becoming too full too soon.
  (you could also change the dimensions of the matrices to a lower value or write the results to a file... )

  Read the help file for information about the implemented functions
  and their limitations/possibilities!!!

  Author: John Merrit (john.merrit@net4all.be)
  Release date: January 30, 1998
  Reader may do with this code whatever he/she wants.

  Disclaimer:
  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

  Brought to you by the letter "S".
  "Progress has no patience - but something's got to give"              }

unit Gen;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Menus, StdCtrls, LinAlg, LinAlgP, ComCtrls;

type
  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    Exit1: TMenuItem;
    ClearMemo1: TMenuItem;
    Close1: TMenuItem;
    Procedures1: TMenuItem;
    Functions1: TMenuItem;
    Decomposition1: TMenuItem;
    RealMatrix1: TMenuItem;
    ComplexMatrix1: TMenuItem;
    RealMatrix2: TMenuItem;
    ComplexMatrix2: TMenuItem;
    RealMatrix3: TMenuItem;
    ComplexMatrix3: TMenuItem;
    InverseMatrix1: TMenuItem;
    Gauss1: TMenuItem;
    InverseMatrix2: TMenuItem;
    Gauss2: TMenuItem;
    LU1: TMenuItem;
    QR1: TMenuItem;
    SingularValue1: TMenuItem;
    LU2: TMenuItem;
    QR2: TMenuItem;
    Determinant1: TMenuItem;
    Rank1: TMenuItem;
    Determinant2: TMenuItem;
    Rank2: TMenuItem;
    Memo1: TRichEdit;
    procedure ClearMemo1Click(Sender: TObject);
    procedure Close1Click(Sender: TObject);
    procedure RealMatrixProcedures(Sender: TObject);
    procedure ComplexMatrixProcedures(Sender: TObject);
    procedure RealMatrixDecomposition(Sender: TObject);
    procedure ComplexMatrixDecomposition(Sender: TObject);
    procedure RealMatrixFunctions(Sender: TObject);
    procedure ComplexMatrixFunctions(Sender: TObject);
  private
    { Private declarations }
    procedure Write_Message(s: string);
    procedure Write_RealMatrix(M: RealMatrix);
    procedure Write_ComplexMatrix(M: ComplexMatrix);
    procedure Write_RealVector(V: RealVector);
    procedure Write_ComplexVector(V: ComplexVector);

  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.Write_Message(s: string);
begin
     Memo1.Lines.add(s);
end;

{ Write a real matrix to the screen, creating a nice readable look. }
procedure TForm1.Write_RealMatrix(M: RealMatrix);
var a,b:integer;
    matt,mats,mattt:string;
begin
    memo1.Enabled:= true;
    mattt:= '';
    for a:= 1  to M.Row do
    begin
    mats:= '[ ';
    for b:= 1 to M.Col do
    begin
    if abs(M.Matrix^[a]^[b]) < 1E-17 then
    begin
    M.Matrix^[a]^[b]:= 0;
    matt:= '0';
    end
    else
    matt:= FloatToStrF(M.Matrix^[a]^[b], ffexponent,8,2);
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    mattt:= '';
    memo1.Lines.add(mats);
    end;
end;

{ Write a complex matrix to the screen, creating a nice readable look. }
procedure TForm1.Write_ComplexMatrix(M: ComplexMatrix);
var a,b:integer;
    matt,mats,mattt:string;
    c,d: extended;
begin
    Memo1.Enabled:= true;
    mattt:= '';
    for a:= 1  to M.Row do
    begin
    mats:= '[ ';
    for b:= 1 to M.Col do
    begin
    c:= M.Matrix^[a]^[b].re;
    if abs(c) < 1E-17 then c:= 0;
    d:= M.Matrix^[a]^[b].im;
    if abs(d) < 1E-17 then d:= 0;
    matt:= '(' + FloatToStr(c) + ';' + FloatToStr(d) + ')';
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    mattt:= '';
    Memo1.Lines.add(mats);
    end;
end;

{ Write a real vector to the screen, creating a nice readable look. }
procedure TForm1.Write_RealVector(V: RealVector);
var a:integer;
    matt,mats,mattt:string;
begin
    Memo1.Enabled:= true;
    mattt:= '';
    mats:= '[ ';
    for a:= 1 to V.Dimension do
    begin
    if abs(V.Vector^[a]) < 1E-10 then
    begin
    V.Vector^[a]:= 0;
    matt:= IntToStr(round(V.Vector^[a]));
    end
    else
    matt:= FloatToStrF(V.Vector^[a], ffexponent,5,2);
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    Memo1.Lines.add(mats);
end;

{ Complex counterpart of above. }
procedure TForm1.Write_ComplexVector(V: ComplexVector);
var a:integer;
    matt,mats,mattt:string;
begin
    Memo1.Enabled:= true;
    mattt:= '';
    mats:= '[ ';
    for a:= 1 to V.Dimension do
    begin
    matt:= '(' + FloatToStr(V.Vector^[a].re) + ';' +  FloatToStr(V.Vector^[a].im) + ')';
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    Memo1.Lines.add(mats);
end;



procedure TForm1.ClearMemo1Click(Sender: TObject);
begin
     if MessageDlg('Delete the contents of this memo?',
     mtInformation, [mbYes, mbNo], 0) = mrYes
     then
     Memo1.Lines.Clear;
end;

procedure TForm1.Close1Click(Sender: TObject);
begin
     Close;
end;

(* Lines between brackets ( { } ) are optional i.e. not nescessary for the test
   (they only swallow lines from the memo ;-) ).
   Note:
   In this demo, I've included the old algorithms (OldInverse) for comparison reasons
   only.
   These algorithms where available in the previous version of ChroMathsome - LinalgPro.
   In the registered version, you will only find the new ones since they have a greater
   accuracy.                   *)
procedure TForm1.RealMatrixProcedures(Sender: TObject);
var M, M1, M2: RealMatrix;
begin
      InitRealMatrix(M, 50, 50);
      InitRealMatrix(M1, 15, 15);
      InitRealMatrix(M2, 5, 5);

      try
      Random_RealMatrix(M, 10000);
      Write_Message('Original matrix: ');
      Write_RealMatrix(M);

      if Sender = InverseMatrix1 then
      begin
      Copy_RealMatrix(M1, M);

      Write_Message('Inverting matrix (old algorithm)... ');
      RealMatrix_OldInverse(M);
      {Write_RealMatrix(M);}
      Write_Message('Done. Testing result ... ');
      RealMatrix_Multiplication(M2, M1, M);
      Write_Message('Done. Test result: ');
      Write_RealMatrix(M2);

      Write_Message('Inverting matrix (new algorithm)... ');
      Copy_RealMatrix(M, M1);
      RealMatrix_Inverse(M);
      {Write_RealMatrix(M);}
      Write_Message('Done. Testing result ... ');
      RealMatrix_Multiplication(M2, M1, M);
      Write_Message('Done. Test result: ');
      Write_RealMatrix(M2);
      { Eventually, use the LinAlg function RealMatrix_AreTheSame to test the similarity: }
      {RealIdentityMatrix(M1);
      if RealMatrix_AreTheSame(M2, M1, 1E-15) = true then
      Write_Message('Round-off error < 1E-15'); }
      end
      else if Sender = Gauss1 then
      begin
      RealMatrix_GaussNormalForm(M);
      Write_Message('Result matrix: ');
      Write_RealMatrix(M);
      end;

      finally
      DestroyRealMatrix(M);
      DestroyRealMatrix(M1);
      DestroyRealMatrix(M2);
      end;
end;

procedure TForm1.ComplexMatrixProcedures(Sender: TObject);
var M, M1, M2: ComplexMatrix;
begin
      InitComplexMatrix(M, 50, 50);
      InitComplexMatrix(M1, 15, 15);
      InitComplexMatrix(M2, 5, 5);

      try
      Random_ComplexMatrix(M, 10000);
      Write_Message('Original matrix: ');
      Write_ComplexMatrix(M);

      if Sender = InverseMatrix2 then
      begin
      Copy_ComplexMatrix(M1, M);

      Write_Message('Inverting matrix (old algorithm)... ');
      ComplexMatrix_OldInverse(M);
      {Write_ComplexMatrix(M);}
      Write_Message('Done. Testing result ... ');
      ComplexMatrix_Multiplication(M2, M1, M);
      Write_Message('Done. Test result: ');
      Write_ComplexMatrix(M2);

      Write_Message('Inverting matrix (new algorithm)... ');
      Copy_ComplexMatrix(M, M1);
      ComplexMatrix_Inverse(M);
      {Write_ComplexMatrix(M);}
      Write_Message('Done. Testing result ... ');
      ComplexMatrix_Multiplication(M2, M1, M);
      Write_Message('Done. Test result: ');
      Write_ComplexMatrix(M2);
      end
      else if Sender = Gauss2 then
      begin
      ComplexMatrix_GaussNormalForm(M);
      Write_Message('Result matrix: ');
      Write_ComplexMatrix(M);
      end;

      finally
      DestroyComplexMatrix(M);
      DestroyComplexMatrix(M1);
      DestroyComplexMatrix(M2);
      end;
end;

{ LU, QR and Singular Value of a matrix. }
procedure TForm1.RealMatrixDecomposition(Sender: TObject);
var M, M1, M2, M3: RealMatrix;
    V: RealVector;
begin
      InitRealMatrix(M, 50, 50);
      InitRealMatrix(M1, 15, 15);
      InitRealMatrix(M2, 5, 5);
      InitRealMatrix(M3, 5, 5);
      InitRealVector(V, 5);
      try
      Random_RealMatrix(M, 10000);
      Write_Message('Original matrix: ');
      Write_RealMatrix(M);

      if Sender = LU1 then
      begin
      Copy_RealMatrix(M3, M); { For testing ... () }
      Write_Message('Decomposing matrix in L & U counterparts ... ');
      RealMatrix_LUDecomposition1(M1, M2, M);
      {Write_Message('Matrix L: ');}
      {Write_RealMatrix(M1);}
      {Write_Message('Matrix U: '); }
      {Write_RealMatrix(M2);}
      Write_Message('Done. Testing result ... ');
      NilRealMatrix(M);
      RealMatrix_Multiplication(M, M1, M2);
      Write_Message('Done. Test result: ');
      Write_RealMatrix(M);
      { ... () test accuracy of decomposition. }
      if RealMatrix_AreTheSame(M, M3, 1E-10) = true then
      Write_Message('Decomposition OK!') else
      Write_Message('Error > 1E-10!');
      end
      else if Sender = QR1 then
      begin
      Copy_RealMatrix(M3, M); { For testing ... () }
      Write_Message('Decomposing matrix in Q & R counterparts ... ');
      RealMatrix_QRDecomposition(M1, M2, M);
      {Write_Message('Matrix Q: ');}
      {Write_RealMatrix(M1);}
      {Write_Message('Matrix R: ');}
      {Write_RealMatrix(M2);}
      Write_Message('Done. Testing result ... ');
      NilRealMatrix(M);
      RealMatrix_Multiplication(M, M1, M2);
      Write_Message('Done. Test result: ');
      Write_RealMatrix(M);
      { ... () test accuracy of decomposition. }
      if RealMatrix_AreTheSame(M, M3, 1E-10) = true then
      Write_Message('Decomposition OK!') else
      Write_Message('Error > 1E-10!');
      end
      else { Singular value }
      begin
      RealMatrix_SingularValueDecomposition(M, M1, V);
      Write_Message('Done. ');
      {Write_Message('Matrix M: '); }
      {Write_RealMatrix(M);          }
      {Write_Message('Matrix M1: '); }
      {Write_RealMatrix(M1);         }
      Write_Message('Singular values of matrix: ');
      Write_RealVector(V);

      Write_Message('Test: ');
      RealDiagonalMatrix(M2, V);
      RealMatrix_Multiplication(M3, M, M2);
      RealMatrix_Transpose(M1);
      RealMatrix_Multiplication(M, M3, M1); { M = U * W * Transpose(V) }
      Write_Message('Matrix M: ');
      Write_RealMatrix(M);
      end;

      finally
      DestroyRealMatrix(M);
      DestroyRealMatrix(M1);
      DestroyRealMatrix(M2);
      DestroyRealMatrix(M3);
      DestroyRealVector(V);
      end;
end;

procedure TForm1.ComplexMatrixDecomposition(Sender: TObject);
var M, M1, M2: ComplexMatrix;
begin
      InitComplexMatrix(M, 50, 50);
      InitComplexMatrix(M1, 15, 15);
      InitComplexMatrix(M2, 5, 5);

      try
      Random_ComplexMatrix(M, 10000);
      Write_Message('Original matrix: ');
      Write_ComplexMatrix(M);

      if Sender = LU2 then
      begin
      Write_Message('Decomposing matrix in L & U counterparts ... ');
      ComplexMatrix_LUDecomposition1(M1, M2, M);
      {Write_Message('Matrix L: '); }
      {Write_ComplexMatrix(M1); }
      {Write_Message('Matrix U: '); }
      {Write_ComplexMatrix(M2); }
      Write_Message('Done. Testing result ... ');
      NilComplexMatrix(M);
      ComplexMatrix_Multiplication(M, M1, M2);
      Write_Message('Done. Test result: ');
      Write_ComplexMatrix(M);
      end
      else if Sender = QR2 then
      begin
      Write_Message('Decomposing matrix in Q & R counterparts ... ');
      ComplexMatrix_QRDecomposition(M1, M2, M);
      {Write_Message('Matrix Q: ');}
      {Write_ComplexMatrix(M1);}
      {Write_Message('Matrix R: ');}
      {Write_ComplexMatrix(M2);}
      Write_Message('Done. Testing result ... ');
      NilComplexMatrix(M);
      ComplexMatrix_Multiplication(M, M1, M2);
      Write_Message('Done. Test result: ');
      Write_ComplexMatrix(M);
      end;

      finally
      DestroyComplexMatrix(M);
      DestroyComplexMatrix(M1);
      DestroyComplexMatrix(M2);
      end;
end;



procedure TForm1.RealMatrixFunctions(Sender: TObject);
var M, M1: RealMatrix;
    det: extended;
    rank, i, j: integer;
begin
      InitRealMatrix(M, 15, 15);
      InitRealMatrix(M1, 15, 15);
      Random_RealMatrix(M, 10000);
      try

      if Sender = Determinant1 then
      begin
      Write_Message('Original matrix: ');
      Write_RealMatrix(M);
      det:= RealMatrix_Determinant(M);
      Write_Message('Determinant of matrix: ' + FloatToStr(det));
      end
      else if Sender = Rank1 then
      begin
      i:= random(10) + 1;
      j:= random(10) + 1;
      Redimension_RealMatrix(M, i, j);
      Write_Message('Original matrix: ');
      Write_RealMatrix(M);
      Copy_RealMatrix(M1, M);
      Write_Message('Original matrix m1: ');
      Write_RealMatrix(M1);
      RealMatrix_HermiteNormalForm(M1);
      Write_Message('Original matrix reduced: ');
      Write_RealMatrix(M1);
      rank:= RealMatrix_Rank(M);
      Write_Message('Rank of matrix: ' + IntToStr(rank));
      rank:= RealMatrix_NumberOfNilRows(M1);
      Write_Message('# of nil rows of matrix: ' + IntToStr(rank));
      end;

      finally
      DestroyRealMatrix(M);
      DestroyRealMatrix(M1);
      end;
end;

procedure TForm1.ComplexMatrixFunctions(Sender: TObject);
var M, M1: ComplexMatrix;
    det: complex;
    rank, i, j: integer;
begin
      InitComplexMatrix(M, 15, 15);
      InitComplexMatrix(M1, 15, 15);
      Random_ComplexMatrix(M, 10000);
      try

      if Sender = Determinant2 then
      begin
      Write_Message('Original matrix: ');
      Write_ComplexMatrix(M);
      det:= ComplexMatrix_Determinant(M);
      Write_Message('Determinant of matrix: (' + FloatToStr(det.re) + ';' + FloatToStr(det.im) + ')');
      end
      else if Sender = Rank2 then
      begin
      i:= random(10) + 1;
      j:= random(10) + 1;
      Redimension_ComplexMatrix(M, i, j);
      Write_Message('Original matrix: ');
      Write_ComplexMatrix(M);
      Copy_ComplexMatrix(M1, M);
      Write_Message('Original matrix m1: ');
      Write_ComplexMatrix(M1);
      ComplexMatrix_HermiteNormalForm(M1);
      Write_Message('Original matrix reduced: ');
      Write_ComplexMatrix(M1);
      rank:= ComplexMatrix_Rank(M);
      Write_Message('Rank of matrix: ' + IntToStr(rank));
      rank:= ComplexMatrix_NumberOfNilRows(M1);
      Write_Message('# of nil rows of matrix: ' + IntToStr(rank));
      end;

      finally
      DestroyComplexMatrix(M);
      DestroyComplexMatrix(M1);
      end;
end;

end.
