{ Demo program for ChroMathsome - Linear Algebra for Sparse Matrices.
  Below, you find example implementations of using functions available in
  ChroMathsome - Linear Algebra for Sparse Matrices.

  This code was written during the development stages of ChroMathsome - Linear Algebra.
  There's no pre-written script followed so you may find (some of) this code
  (the order of implemented functions) and some of the variables a little strange).

  Note that, since ChroMathsome has its own pre-defined exceptions, you do not need to
  write supersofisticated exception handling routines.
  The only exceptions written here are to make sure the pointers get freed, even
  when an exception has occured (a try ... finally block).

  Since ChroMathsome - Linear Algebra automatically updates the dimensions of
  matrices and vectors (if needed), we gave most of the matrices/vectors in the
  initialization part (InitRealMatrix, ...) a dummy value.
  You are, however, encouraged to enter the correct values (if you know them at forehand)
  since this will make your code much more readable!

  Read the help file for information about the implemented functions
  and their limitations/possibilities!!!

  Author: John Merrit (john.merrit@net4all.be)
  Release date: January 30, 1998
  Reader may do with this code whatever he/she wants.

  Disclaimer:
  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

  Brought to you by the letter "E".
  "We're only astronauts in the weightlessness of a pixellated space
   Analogue species in a digital world"              }

unit Algsdemo;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, LinAlg, LinAlgS, Menus;

type
  TForm1 = class(TForm)
    Memo1: TMemo;
    MainMenu1: TMainMenu;
    Exit1: TMenuItem;
    ClearMemo1: TMenuItem;
    Close1: TMenuItem;
    Special1: TMenuItem;
    Diagonal1: TMenuItem;
    TriDiagonal1: TMenuItem;
    Convert1: TMenuItem;
    SparsetoNormal1: TMenuItem;
    SortElements1: TMenuItem;
    Building1: TMenuItem;
    AddDeleteElement1: TMenuItem;
    GetRow1: TMenuItem;
    GetColumn1: TMenuItem;
    Procedures1: TMenuItem;
    Real1: TMenuItem;
    Complex1: TMenuItem;
    Transpose1: TMenuItem;
    Norm1: TMenuItem;
    Norm2: TMenuItem;
    Transpose2: TMenuItem;
    Conjugate1: TMenuItem;
    Interactive1: TMenuItem;
    GetMatrix1: TMenuItem;
    Arithmetic1: TMenuItem;
    AddtwoMatrices1: TMenuItem;
    MultiplytwoMatrices1: TMenuItem;
    procedure ClearMemo1Click(Sender: TObject);
    procedure Close1Click(Sender: TObject);
    procedure Special(Sender: TObject);
    procedure SparsetoNormal1Click(Sender: TObject);
    procedure SortElements1Click(Sender: TObject);
    procedure AddDeleteElement1Click(Sender: TObject);
    procedure RealProcedures(Sender: TObject);
    procedure ComplexProcedures(Sender: TObject);
    procedure GetMatrix1Click(Sender: TObject);
    procedure Arithm(Sender: TObject);
  private
    { Private declarations }
    procedure Write_Message(s: string);
    procedure Write_RealMatrix(M: RealMatrix);
    procedure Write_ComplexMatrix(M: ComplexMatrix);
    procedure Write_SparseRealMatrix(M: SparseRealMatrix);
    procedure Write_SparseComplexMatrix(M: SparseComplexMatrix);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses SMDialog;
{$R *.DFM}

procedure TForm1.Write_Message(s: string);
begin
     Memo1.Lines.add(s);
end;

{ Write a real matrix to the screen, creating a nice readable look. }
procedure TForm1.Write_RealMatrix(M: RealMatrix);
var a,b:integer;
    matt,mats,mattt:string;
begin
    memo1.Enabled:= true;
    mattt:= '';
    for a:= 1  to M.Row do
    begin
    mats:= '[ ';
    for b:= 1 to M.Col do
    begin
    if abs(M.Matrix^[a]^[b]) < 1E-17 then
    begin
    M.Matrix^[a]^[b]:= 0;
    matt:= '0';
    end
    else
    matt:= FloatToStrF(M.Matrix^[a]^[b], ffexponent,8,2);
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    mattt:= '';
    memo1.Lines.add(mats);
    end;
end;

{ Write a complex matrix to the screen, creating a nice readable look. }
procedure TForm1.Write_ComplexMatrix(M: ComplexMatrix);
var a,b:integer;
    matt,mats,mattt:string;
    c,d: extended;
begin
    Memo1.Enabled:= true;
    mattt:= '';
    for a:= 1  to M.Row do
    begin
    mats:= '[ ';
    for b:= 1 to M.Col do
    begin
    c:= M.Matrix^[a]^[b].re;
    if abs(c) < 1E-17 then c:= 0;
    d:= M.Matrix^[a]^[b].im;
    if abs(d) < 1E-17 then d:= 0;
    matt:= '(' + FloatToStr(c) + ';' + FloatToStr(d) + ')';
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    mattt:= '';
    Memo1.Lines.add(mats);
    end;
end;

{ Merely for testing. }
procedure TForm1.Write_SparseRealMatrix(M: SparseRealMatrix);
var i:integer;
    s: string;
begin
    memo1.Enabled:= true;
    for i:= 1 to M.Dimension do
    begin
    s:= 'Element ' + IntToStr(i);
    s:= s + ' Row: ' + IntToStr(M.Matrix^[i].Row);
    s:= s + ' Col: ' + IntToStr(M.Matrix^[i].Col);
    s:= s + ' Value: ' + FloatToStr(M.Matrix^[i].Value);
    Write_Message(s);
    s:= '';
    end;
end;

procedure TForm1.Write_SparseComplexMatrix(M: SparseComplexMatrix);
var i:integer;
    s: string;
begin
    memo1.Enabled:= true;
    for i:= 1 to M.Dimension do
    begin
    s:= 'Element ' + IntToStr(i);
    s:= s + ' Row: ' + IntToStr(M.Matrix^[i].Row);
    s:= s + ' Col: ' + IntToStr(M.Matrix^[i].Col);
    s:= s + ' Value: (' + FloatToStr(M.Matrix^[i].Value.re) + ';' + FloatToStr(M.Matrix^[i].Value.im) + ')';
    Write_Message(s);
    s:= '';
    end;
end;

procedure TForm1.ClearMemo1Click(Sender: TObject);
begin
     if MessageDlg('Delete the contents of this memo?',
     mtInformation, [mbYes, mbNo], 0) = mrYes
     then
     Memo1.Lines.Clear;
end;

procedure TForm1.Close1Click(Sender: TObject);
begin
     Close;
end;

procedure TForm1.Special(Sender: TObject);
var MS: SparseRealMatrix;
    MR: RealMatrix;
    i, sr, sc: integer;
    x: RealMatrixCell;
begin
     InitSparseRealMatrix(MS, 10);
     InitRealMatrix(MR, 1, 1);
     try
     { Create a sparse 10*10 diagonal matrix with random elements. }
     if Sender = Diagonal1 then
     begin
     for i:= 1 to 10 do
     begin
     sr:= i;
     sc:= i;
     MS.Matrix^[i].Row:= sr;
     MS.Matrix^[i].Col:= sc;
     MS.Matrix^[i].Value:= random(100) + 1;
     end;
     { Convert the sparse matrix to a (normal) real matrix. }
     SparseRealMatrixToRealMatrix(MR, MS);
     Write_Message('Matrix: ');
     Write_RealMatrix(MR);
     Write_Message('Trace of matrix: ' + FloatToStr(SparseRealMatrix_Trace(MS)));
     end
     else { Create a sparse tri-diagonal 10*10 matrix with random elements. }
     begin
     for i:= 1 to 10 do  { Fill in main diagonal }
     begin
     sr:= i;
     sc:= i;
     MS.Matrix^[i].Row:= sr;
     MS.Matrix^[i].Col:= sc;
     MS.Matrix^[i].Value:= random(100) + 1;
     if (i < 10) then  { Fill in other two diagonals }
     begin
     x.Row:= i;
     x.Col:= i + 1;
     x.Value:= random(100) + 1;
     SparseRealMatrix_AddElement(MS, x);
     end;
     if (i > 1) then
     begin
     x.Row:= i;
     x.Col:= i - 1;
     x.Value:= random(100) + 1;
     SparseRealMatrix_AddElement(MS, x);
     end;
     end;
     { Convert the sparse matrix to a (normal) real matrix. }
     SparseRealMatrixToRealMatrix(MR, MS);
     Write_Message('Matrix: ');
     Write_RealMatrix(MR);
     end;
     finally
     DestroySparseRealMatrix(MS);
     DestroyRealMatrix(MR);
     end;
end;

procedure TForm1.SparsetoNormal1Click(Sender: TObject);
var MS: SparseRealMatrix;
    MSC: SparseComplexMatrix;
    MR: RealMatrix;
    MC: ComplexMatrix;
    i, sr, sc: integer;
begin
     InitSparseRealMatrix(MS, 30);
     InitRealMatrix(MR, 1, 1);
     InitSparseComplexMatrix(MSC, 30);
     InitComplexMatrix(MC, 1, 1);
     try
     { Create a sparse matrix with random elements (and rows and columns). }
     for i:= 1 to 30 do
     begin
     sr:= random(10) + 1;
     sc:= random(30) + 1;
     MS.Matrix^[i].Row:= sr;
     MS.Matrix^[i].Col:= sc;
     MS.Matrix^[i].Value:= random(100) + 1;
     end;
     Write_Message('Sparse matrix: ');
     Write_SparseRealMatrix(MS);
     { Sort elements, delete doubles. }
     SparseRealMatrix_SortElements(MS);
     Write_Message('Sorted sparse matrix: ');
     Write_SparseRealMatrix(MS);
     { Convert the sparse matrix to a (normal) real matrix. }
     SparseRealMatrixToRealMatrix(MR, MS);
     Write_Message('Real matrix: ');
     Write_RealMatrix(MR);
     { Convert the real matrix back to a sparse matrix.
       The 1E-15 defines the smallest value that must be interpreted as non-zero.
       Note that you will get an error if your matrix does not contain atleast 1 element
       that is smaller than this value.                                    }
     RealMatrixToSparseRealMatrix(MS, MR, 1E-15);
     { Test }
     SparseRealMatrixToRealMatrix(MR, MS);
     Write_Message('Test ... Matrix: ');
     Write_RealMatrix(MR);

     { Same for a complex matrix. }
     for i:= 1 to 30 do
     begin
     sr:= random(10) + 1;
     sc:= random(30) + 1;
     MSC.Matrix^[i].Row:= sr;
     MSC.Matrix^[i].Col:= sc;
     MSC.Matrix^[i].Value.re:= random(100) + 1;
     MSC.Matrix^[i].Value.im:= random(100) + 1;
     end;
     Write_Message('Sparse matrix: ');
     Write_SparseComplexMatrix(MSC);
     SparseComplexMatrix_SortElements(MSC);
     Write_Message('Sorted sparse matrix: ');
     Write_SparseComplexMatrix(MSC);
     SparseComplexMatrixToComplexMatrix(MC, MSC);
     Write_Message('Complex matrix: ');
     Write_ComplexMatrix(MC);
     ComplexMatrixToSparseComplexMatrix(MSC, MC, 1E-15);
     { Test }
     SparseComplexMatrixToComplexMatrix(MC, MSC);
     Write_Message('Test ... Matrix: ');
     Write_ComplexMatrix(MC);
     finally
     DestroySparseRealMatrix(MS);
     DestroyRealMatrix(MR);
     DestroySparseComplexMatrix(MSC);
     DestroyComplexMatrix(MC);
     end;
end;

procedure TForm1.SortElements1Click(Sender: TObject);
var MS: SparseRealMatrix;
    i, sr, sc: integer;
begin
     InitSparseRealMatrix(MS, 30);
     try
     { Create a sparse matrix with random elements (and rows and columns). }
     for i:= 1 to 30 do
     begin
     sr:= random(5) + 1;
     sc:= random(5) + 1;
     MS.Matrix^[i].Row:= sr;
     MS.Matrix^[i].Col:= sc;
     MS.Matrix^[i].Value:= random(100) + 1;
     end;
     Write_Message('Sparse Matrix: ');
     Write_SparseRealMatrix(MS);
     { Sort the elements. }
     SparseRealMatrix_SortElements(MS);
     Write_Message('Sparse Matrix: ');
     Write_SparseRealMatrix(MS);
     finally
     DestroySparseRealMatrix(MS);
     end;
end;

procedure TForm1.AddDeleteElement1Click(Sender: TObject);
var MS, MS2: SparseRealMatrix;
    MR: RealMatrix;
    i, sr, sc: integer;
    Element: RealMatrixCell;  { Read help file }
begin
     InitSparseRealMatrix(MS, 30);
     InitSparseRealMatrix(MS2, 1);
     InitRealMatrix(MR, 1, 1);
     try
     { Create a sparse matrix with random elements (and rows and columns). }
     for i:= 1 to 30 do
     begin
     sr:= random(5) + 1;
     sc:= random(30) + 1;
     MS.Matrix^[i].Row:= sr;
     MS.Matrix^[i].Col:= sc;
     MS.Matrix^[i].Value:= random(100);
     end;
     { Sort elements, delete doubles. }
     SparseRealMatrix_SortElements(MS);
     { Convert the sparse matrix to a (normal) real matrix. }
     SparseRealMatrixToRealMatrix(MR, MS);
     Write_Message('Matrix: ');
     Write_RealMatrix(MR);
     if Sender = AddDeleteElement1 then{ Add a new element to the sparse matrix                             }
     begin
     Element.Row:= 15;
     Element.Col:= 8;
     Element.Value:= 666;
     SparseRealMatrix_AddElement(MS, Element);
     { Test }
     SparseRealMatrixToRealMatrix(MR, MS);
     Write_Message('New Matrix: ');
     Write_RealMatrix(MR);
     { Delete the element again   }
     SparseRealMatrix_DeleteElement(MS, Element);
     { Test }
     SparseRealMatrixToRealMatrix(MR, MS);
     Write_Message('Original Matrix: ');
     Write_RealMatrix(MR);
     end
     else if Sender = GetRow1 then  { Return all (non-zero) elements in a row. }
     begin
     for i:= 1 to 5 do
     begin
     SparseRealMatrix_GetRow(MS2, MS, i);
     Write_Message('Matrix elements of row ' + IntToStr(i) + ':');
     Write_SparseRealMatrix(MS2);
     end;
     end
     else if Sender = GetColumn1 then { Return all (non-zero) elements in column 1. }
     begin
     SparseRealMatrix_Transpose(MS);
     SparseRealMatrix_SortElements(MS); (* Sort elements *)
     SparseRealMatrix_GetRow(MS2, MS, 1);
     Write_Message('Matrix elements of column 1');
     Write_SparseRealMatrix(MS2);
     end;
     finally
     DestroySparseRealMatrix(MS);
     DestroySparseRealMatrix(MS2);
     DestroyRealMatrix(MR);
     end;
end;

procedure TForm1.RealProcedures(Sender: TObject);
var MS: SparseRealMatrix;
    MR: RealMatrix;
    i, sr, sc: integer;
    x: extended;
begin
     InitSparseRealMatrix(MS, 10);
     InitRealMatrix(MR, 1, 1);
     try
     { Create a sparse matrix with random elements (and rows and columns). }
     for i:= 1 to 10 do
     begin
     sr:= random(5) + 1;
     sc:= random(15) + 1;
     MS.Matrix^[i].Row:= sr;
     MS.Matrix^[i].Col:= sc;
     MS.Matrix^[i].Value:= random(100);
     end;
     SparseRealMatrix_SortElements(MS); { Delete possible double elements. }
     { Convert the sparse matrix to a (normal) real matrix. }
     SparseRealMatrixToRealMatrix(MR, MS);
     Write_Message('Matrix: ');
     Write_RealMatrix(MR);

     if Sender = Transpose1 then
     begin
     {  Transpose sparse matrix. }
     SparseRealMatrix_Transpose(MS);
     { Test }
     SparseRealMatrixToRealMatrix(MR, MS);
     Write_Message('Transposed Matrix: ');
     Write_RealMatrix(MR);
     end
     else { Calculate norm of sparse matrix. }
     begin
     x:= SparseRealMatrix_Norm(MS);
     Write_Message('Norm of Matrix: ' + FloatToStr(x));
     end;
     finally
     DestroySparseRealMatrix(MS);
     DestroyRealMatrix(MR);
     end;
end;

procedure TForm1.ComplexProcedures(Sender: TObject);
var MS: SparseComplexMatrix;
    MR: ComplexMatrix;
    i, sr, sc: integer;
    x: complex;
begin
     InitSparseComplexMatrix(MS, 10);
     InitComplexMatrix(MR, 1, 1);
     try
     { Create a sparse matrix with random elements (and rows and columns). }
     for i:= 1 to 10 do
     begin
     sr:= random(5) + 1;
     sc:= random(15) + 1;
     MS.Matrix^[i].Row:= sr;
     MS.Matrix^[i].Col:= sc;
     MS.Matrix^[i].Value.re:= random(100);
     MS.Matrix^[i].Value.im:= random(100);
     end;
     SparseComplexMatrix_SortElements(MS); { Delete possible double elements. }
     { Convert the sparse matrix to a (normal) Complex matrix. }
     SparseComplexMatrixToComplexMatrix(MR, MS);
     Write_Message('Matrix: ');
     Write_ComplexMatrix(MR);

     if Sender = Transpose2 then
     begin
     {  Transpose sparse matrix. }
     SparseComplexMatrix_Transpose(MS);
     { Test }
     SparseComplexMatrixToComplexMatrix(MR, MS);
     Write_Message('Transposed Matrix: ');
     Write_ComplexMatrix(MR);
     end
     else if Sender = Conjugate1 then
     begin
     {  Transpose sparse matrix. }
     SparseComplexMatrix_Conjugate(MS);
     { Test }
     SparseComplexMatrixToComplexMatrix(MR, MS);
     Write_Message('Conjugate Matrix: ');
     Write_ComplexMatrix(MR);
     end
     else { Calculate norm of sparse matrix. }
     begin
     x:= SparseComplexMatrix_Norm(MS);
     Write_Message('Norm of Matrix: (' + FloatToStr(x.re) + ';' + FloatTostr(x.im) + ')');
     end;
     finally
     DestroySparseComplexMatrix(MS);
     DestroyComplexMatrix(MR);
     end;
end;

{ Retrieve matrix data from user.                  }
procedure TForm1.GetMatrix1Click(Sender: TObject);
var MS: SparseRealMatrix;
    Element: RealMatrixCell;
    i: integer;
    x: extended;
begin
     InitSparseRealMatrix(MS, 1);
     try
     { Create a sparse matrix from user input. }
     if SparseMatrixDlg.ShowModal = mrOK then
     begin
     repeat
     begin
     i:= StrToInt(SparseMatrixDlg.RowEdit.Text);
     if i <= 0 then Element.Row:= 1 else Element.Row:= i;
     i:= StrToInt(SparseMatrixDlg.ColEdit.Text);
     if i <= 0 then Element.Col:= 1 else Element.Col:= i;
     x:= StrToFloat(SparseMatrixDlg.ValueEdit.Text);
     Element.Value:= x;

     { Check if matrix already contains an element with same row and col values.
       If so, let user know else add it to the sparse matrix.
       Note that, by initialization of MS, already one element (MS.Matrix^[1]) has a
       row and col value of 1.
       Therefore, in real life applications, bypass the Does_ElementExists test
       for the first element the user enters.
       Also note that, the last element the user enters will be ignored.
       Why? Study the code in this loop you're reading, it isn't perfectly implemented
        --> this is only a demonstration! }
     if SparseRealMatrix_DoesElementExists(MS, Element) = true then
     ShowMessage('I already have an element in row ' + IntToStr(Element.Row) +
                 ' and column ' + IntToStr(Element.Col))
     else SparseRealMatrix_AddElement(MS, Element);

     SparseMatrixDlg.ShowModal;
     end;
     until SparseMatrixDlg.RadioGroup1.ItemIndex = 1;
     end;

     Write_Message('Your matrix: ');
     Write_SparseRealMatrix(MS);

     finally
     DestroySparseRealMatrix(MS);
     end;
end;

procedure TForm1.Arithm(Sender: TObject);
var MS, MS1, MS2: SparseRealMatrix;
    MR, M1, M2: RealMatrix;
    i, sr, sc: integer;
    Element: RealMatrixCell;  { Read help file }
begin
     InitSparseRealMatrix(MS1, 10);
     InitSparseRealMatrix(MS2, 10);
     InitSparseRealMatrix(MS, 1);
     InitRealMatrix(MR, 1, 1);
     InitRealMatrix(M1, 1, 1);
     InitRealMatrix(M2, 1, 1);
     try
     { Create a sparse matrix with random elements (and rows and columns). }
     for i:= 1 to 10 do
     begin
     sr:= random(3) + 1;
     sc:= random(3) + 1;
     MS1.Matrix^[i].Row:= sr;
     MS1.Matrix^[i].Col:= sc;
     MS1.Matrix^[i].Value:= random(10);
     end;
     SparseRealMatrix_SortElements(MS1); { Delete possible double elements. }
     { Convert the sparse matrix to a (normal) real matrix. }
     SparseRealMatrixToRealMatrix(M1, MS1);
     Write_Message('Matrix 1: ');
     Write_RealMatrix(M1);

     for i:= 1 to 10 do
     begin
     sr:= random(3) + 1;
     sc:= random(3) + 1;
     MS2.Matrix^[i].Row:= sr;
     MS2.Matrix^[i].Col:= sc;
     MS2.Matrix^[i].Value:= random(10);
     end;
     SparseRealMatrix_SortElements(MS2); { Delete possible double elements. }
     { Convert the sparse matrix to a (normal) real matrix. }
     SparseRealMatrixToRealMatrix(M2, MS2);
     Write_Message('Matrix 2: ');
     Write_RealMatrix(M2);

     if Sender = MultiplyTwoMatrices1 then
     begin
     SparseRealMatrix_Multiplication(MS, MS1, MS2);

     { Test }
     SparseRealMatrixToRealMatrix(MR, MS);
     Write_Message('New Matrix: ');
     Write_RealMatrix(MR);
     RealMatrix_Multiplication(MR, M1, M2);
     Write_Message('Test: ');
     Write_RealMatrix(MR);
     end
     else if Sender = AddTwoMatrices1 then
     begin
     SparseRealMatrix_Addition(MS, MS1, MS2);
     { Test }
     SparseRealMatrixToRealMatrix(MR, MS);
     Write_Message('New Matrix: ');
     Write_RealMatrix(MR);
     end;
     finally
     DestroySparseRealMatrix(MS1);
     DestroySparseRealMatrix(MS2);
     DestroySparseRealMatrix(MS);
     DestroyRealMatrix(MR);
     DestroyRealMatrix(M1);
     DestroyRealMatrix(M2);
     end;
end;

end.
